from gconfigs.gconfigs import NOTSET


class DummyBackend:
    """DummyBackend for testing GConfigs
    I'm implementing a immutable dict to store configs.
    This is because I don't want `GConfigs` to ever change values into the
    backend, all the trimming or whatever must be done in `GConfigs` class.

    So if at some point we implement something that changes data in any backend,
    it will fail.
    """

    def __init__(self):
        # ref: https://www.python.org/dev/peps/pep-0351/

        class ImmutableDict(dict):

            def __hash__(self):
                return id(self)

            def _immutable(self, *args, **kws):
                raise TypeError("object is immutable")

            __setitem__ = _immutable
            __delitem__ = _immutable
            clear = _immutable
            update = _immutable
            setdefault = _immutable
            pop = _immutable
            popitem = _immutable

        self.data = ImmutableDict(
            {
                "CONFIG-1": "config-1",
                " white space key ": " white space value ",
                "empty-value": "",
                "space-only-value": "  ",
                "CONFIG-NONE": None,
                "CONFIG-TRUE": True,
                "CONFIG-FALSE": False,
                "CONFIG-INT": 1,
                "CONFIG-FLOAT": 1.1,
                "CONFIG-LIST": [1, 1.1, "a"],
                "CONFIG-TUPLE": (1, 1.1, "a"),
                "CONFIG-DICT": {"a": 1, "b": "b"},
                "CONFIG-TRUE-STRING": "True",
                "CONFIG-FALSE-STRING": "False",
                "CONFIG-LIST-STRING-JSON-STYLE": '[1, 1.1, "a"]',
                "CONFIG-LIST-STRING-JSON-STYLE-BROKEN": '[1, 1.1, "a]',
                "CONFIG-DICT-JSON-STYLE": '{"a": 1, "b": "b"}',
            }
        )

    def keys(self):
        """
        :returns: from iterable `self.data` (an immutable dict) return its keys.
        """
        return self.data

    def get(self, key, **kwargs):
        """ Return value given a key
        :returns: value or KeyError
        """
        value = self.data.get(key, NOTSET)
        if value is NOTSET:
            raise KeyError(
                f"Dummy Data '{key}' not set. Check for any "
                "misconfiguration or misspelling of the variable name."
            )

        return value



# -*- coding: utf-8 -*-

"""
gConfigs - Config and Secret parser
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Nothing shows better than some snippets:

    from gconfigs import envs, configs, secrets

    HOME = envs('HOME')
    DEBUG = configs.as_bool('DEBUG', default=False)
    DATABASE_USER = configs('DATABASE_USER')
    DATABASE_PASS = secrets('DATABASE_PASS')

    >>> # envs, configs and secrets are iterables
    >>> from gconfigs import envs
    >>> for env in envs:
    ...     print(env)
    ...     print(env.key)
    ...     print(env.value)
    ...
    EnvironmentVariable(key='ENV_TEST', value='env-test-1')
    ENV_TEST
    env-test-1
    ...

    >>> 'ENV_TEST' in envs
    True

    >>> envs.json()
    '{"ENV_TEST": "env-test-1", "HOME": "/root", ...}'
"""

__author__ = """Douglas Miranda"""
__email__ = "douglasmirandasilva@gmail.com"
__version__ = "0.1.2"


from .gconfigs import GConfigs
from .backends import LocalEnv, DotEnv, LocalMountFile


envs = GConfigs(backend=LocalEnv, object_type_name="EnvironmentVariable")
dotenvs = GConfigs(backend=DotEnv, object_type_name="DotEnvConfig")
configs = GConfigs(
    backend=LocalMountFile(root_dir="/run/configs"), object_type_name="Config"
)
secrets = GConfigs(
    backend=LocalMountFile(root_dir="/run/secrets"), object_type_name="Secret"
)



# -*- coding: utf-8 -*-
from collections import namedtuple
from functools import lru_cache
import json


class NoValue:

    def __repr__(self):  # pragma: no cover
        return f"<{self.__class__.__name__}>"


NOTSET = NoValue()


class GConfigs:

    def __init__(self, backend, strip=True, object_type_name="KeyValue"):
        """
        :param backend: Backend / parser of configs. A simple class implementing `get` and `keys` methods. `gconfigs.backends` for more information.
        :param strip: Control the stripping of return value of `self.get` method.
        :param object_type_name: Simply a nice name for our key value named tuple.
        """
        if hasattr(backend, "get") or hasattr(backend, "keys"):
            self._backend = backend() if callable(backend) else backend
        else:
            raise AttributeError(
                "'backend' class must have at least the methods 'get' and 'keys'."
            )

        self.strip = strip
        self.object_type_name = object_type_name

        if hasattr(self._backend, "load_file") and callable(self._backend.load_file):
            self.load_file = self._load_file

        self._iter_configs = self.iterator()

    def get(self, key, default=NOTSET, strip=None, **kwargs):
        """Return value for given key.
        :param var: Key (Name) of config.
        :param default: If backend doesn't return valid config, return this instead.
        :param strip: Control the stripping of return value. Override the default `self.strip` with `True` or `False`. Will strip if is a string value.

        :returns: Parsed value or default. Or raises exceptions you implement in your backend.
        """
        try:
            value = self._backend.get(key)
        except Exception as e:
            # This may seem a generic try/except but I'm actually catching the
            # specific Exception that you will implement in your backend.
            if default is NOTSET:
                raise e

            return default

        strip_ = self.strip if strip is None else strip
        if strip_ and isinstance(value, str):
            return value.strip()

        return value

    def as_bool(self, key, **kwargs):
        value = self.get(key, **kwargs)
        if isinstance(value, bool):
            return value

        if isinstance(value, str) and value.lower() in ("true", "false"):
            return self._cast(value.lower())

        raise ValueError(f"Could not cast the value '{value}' to boolean.")

    def as_list(self, key, **kwargs):
        value = self.get(key, **kwargs)
        if isinstance(value, list) or isinstance(value, tuple):
            return list(value)

        if isinstance(value, str) and value.startswith("[") and value.endswith("]"):
            return self._cast(value)

        raise ValueError(f"Could not cast the value '{value}' to list.")

    def as_dict(self, key, **kwargs):
        value = self.get(key, **kwargs)
        if isinstance(value, dict):
            return value

        if isinstance(value, str) and value.startswith("{") and value.endswith("}"):
            return self._cast(value)

        raise ValueError(f"Could not cast the value '{value}' to dict.")

    def _cast(self, value):
        try:
            return json.loads(value)

        except json.decoder.JSONDecodeError as e:
            raise ValueError(
                f"Could not cast the value {value}. Tried to cast with json module, so it must be a valid json value."
            )

    def json(self):
        """Returns json parsed data of all available data.
        """
        return json.dumps({item.key: item.value for item in self.iterator()})

    def _load_file(self, filepath):
        self._backend.load_file(filepath)

    @property
    @lru_cache()
    def _cached_namedtuple(self):
        return namedtuple(self.object_type_name, ["key", "value"])

    def iterator(self):
        for key in self._backend.keys():
            yield self._cached_namedtuple(key, self.get(key))

    def __next__(self):
        return next(self._iter_configs)

    def __iter__(self):
        return self

    def __call__(self, key, **kwargs):
        return self.get(key, **kwargs)

    def __contains__(self, key):
        return key in self._backend.keys()

    def __len__(self):
        return len(self._backend.keys())

    def __repr__(self):  # pragma: no cover
        return f"<GConfigs backend={self._backend.__class__.__name__}>"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
import sys

assert sys.version_info >= (3, 6, 0), "black requires Python 3.6+"

from pathlib import Path

readme = Path("README.rst").read_text()
history = Path("HISTORY.rst").read_text()

setup(
    name="gconfigs",
    version="0.1.2",
    description="gConfigs - Config and Secret parser for your Python projects.",
    long_description=f"{readme}\n\n{history}",
    author="Douglas Miranda",
    author_email="douglasmirandasilva@gmail.com",
    url="https://github.com/douglasmiranda/gconfigs",
    packages=find_packages(include=["gconfigs"]),
    include_package_data=True,
    license="MIT license",
    zip_safe=False,
    keywords="gconfigs configs environment secrets dotenv",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
    ],
)
